/*
 * bst.c
 *
 *  Created on: Nov 5, 2013
 *      Author: mtk
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "bst.h"

bst_element dummy;
void bst_init(bst* t){
	t->root = NULL;
	memset(&dummy,0,sizeof(bst_element));
}

static bst_element error(char* msg){
	fflush(stdout);
	fprintf(stderr,"%s\n",msg);
	return dummy;
}

static int compare(bst_element a, bst_element b){
	if(a.rain > b.rain) return 1;
	if(a.rain < b.rain) return -1;
	return 0;
}



bst_element bst_minimum(bst* t){
	if( t->root == NULL ){
		return error("Empty BST.");
	}
	bst_node* current = t->root;
	while( current->left != NULL){
		current = current->left;
	}
	return current->key;
}


bst_element bst_maximum(bst* t){
	if( t->root == NULL ){
		return error("Empty BST.");;
	}
	bst_node* current = t->root;
	while( current->right != NULL){
		current = current->right;
	}
	return current->key;
}

bst_node* search_node(bst* t, bst_element e){
	if( t->root == NULL ){
		return NULL;
	}
	bst_node* current = t->root;
	while( current!=NULL && compare(current->key,e)!=0){
		if( compare(e ,current->key) < 0 ) current = current->left;
//		if( e  < current->key ) current = current->left;
		else current = current->right;
	}
	return current;
}

bst_element bst_search(bst* t, bst_element e){
	bst_node* current = search_node(t,e);
	if(current) return current->key;
	else return error("Element not found.");
}

bst_node* predecessor_node(bst_node* n){
	if( n->left != NULL){
		bst_node* current = n->left;
		while( current->right != NULL){
			current = current->right;
		}
		return current;
	}
	bst_node* p=n->parent;
	while( p!=NULL && n != p->right){
		n=p;
		p=p->parent;
	}
	return p;
}

bst_element bst_predecessor(bst* t, bst_element e){
	bst_node* n = search_node(t,e);
	if(n==NULL){
		return error("Error: cannot find specified element.");
	}
	n = predecessor_node(n);
	if(n==NULL){
		return error("Error: cannot find predecessor in BST.");
	}
	return n->key;
}

bst_node* successor_node(bst_node* n){
	if( n->right != NULL){
		bst_node* current = n->right;
		while( current->left != NULL){
			current = current->left;
		}
		return current;
	}
	bst_node* p=n->parent;
	while( p!=NULL && n != p->left){
		n=p;
		p=p->parent;
	}
	return p;
}
bst_element bst_successor(bst* t, bst_element e){
	bst_node* n = search_node(t,e);
	if(n==NULL){
		return error("Error: cannot find specified element.");
	}
	n = successor_node(n);
	if(n==NULL){
		return error("Error: cannot find successor in BST.");
	}
	return n->key;
}

/* insert operations */
bst_node* bst_insert(bst* t, bst_element e){
	bst_node* n = malloc(sizeof(bst_node));
	n->key = e;
	n->parent = NULL;
	n->left = NULL;
	n->right = NULL;

	bst_node* p = NULL;
	bst_node* c = t->root;

	if(c==NULL){
		t->root = n;
		return n;
	}

	while(c!=NULL){
		p=c;
		if(compare(e ,c->key) < 0) c=c->left;
		else c=c->right;
	}
	n->parent = p;
	if( compare(e ,p->key) < 0 ) p->left = n;
	else p->right = n;

	return n;
}

void bst_delete(bst* t, bst_element e){

}

