/*
 * hashtable.c
 *
 *  Created on: Oct 24, 2013
 *      Author: mtk
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "hashtable.h"



typedef
struct hash_el_tag {
	key_type key;
	value_type value;
	struct hash_el_tag* next;
} hash_el;

#define m 77
hash_el* table[m];
int n;


static long h(char *key){
	long sum = 0;
	int offs = 0;
	while(*key!='\0'){
		sum+=(*key) << offs;
		offs = (offs + 8) % 64;
		++key;
	}
	return sum % m;
}

void ht_init(){
	long i;
	for(i=0;i<m;++i){
		table[i]=NULL;
	}
	n=0;
}




void ht_insert(key_type key, value_type value){
    long index = h(key);   // compute the index from the key
    hash_el* p=table[index];
    /* 1. check if the element already exists */
    while(p!=NULL){
    	  if(strcmp(key,p->key)==0){ /* already present => replace the value */
    		  p->value = value;
    		  return;
    	  }
    	  p=p->next;
    }
    /* 2. the key is not yet present in the hash table => insert it */
    p = (hash_el*)malloc(sizeof(hash_el));
    p->key=strdup(key);
    p->value = value;
    p->next = table[index];
    table[index] = p;
    n++;
}

value_type ht_search(key_type key){
    int index = h(key);
    hash_el* p=table[index];
    while(p!=NULL){
    	  if(strcmp(key,p->key)==0){ /* found */
    		  return p->value;
    	  }
    	  p=p->next;
    }
    return 0;
}

int ht_contains_key(key_type key){
    int index = h(key);
    hash_el* p=table[index];
    while(p!=NULL){
    	  if(strcmp(key,p->key)==0){ /* found */
    		  return 1;
    	  }
    	  p=p->next;
    }
    return 0;
}

void ht_delete(key_type key){
    int index = h(key);
    hash_el** p=&table[index];
    while(*p!=NULL){
    	  if(strcmp(key,(*p)->key)==0){ /* found */
    		  hash_el* tbr = *p;
    		  *p = (*p)->next;
    		  free(tbr->key);
    		  free(tbr);
    		  n--;
    		  return;
    	  }
    	  p=&((*p)->next);
    }
    return;
}

void print_ht_table(){
	int i;
	for(i=0; i<m; ++i){
		hash_el* p = table[i];
		while(p!=NULL){
//			printf("%s\n",p->key,p->value);
			p=p->next;
		}
	}
}

int ht_size(){
	return n;
}

key_type* ht_keys(){
	key_type* res = (key_type*)malloc(sizeof(key_type)*n);
	key_type* current=res;
	int i;
	for(i=0; i<m; ++i){
		hash_el* p = table[i];
		while(p!=NULL){
			*(current++) = p->key;
			p=p->next;
		}
	}
	return res;
}

entry* ht_entries(){
	entry* res = (entry*)malloc(sizeof(entry)*n);
	entry* current=res;
	int i;
	for(i=0; i<m; ++i){
		hash_el* p = table[i];
		while(p!=NULL){
			current->key = p->key;
			current->value = p->value;
			current++;
			p=p->next;
		}
	}
	return res;
}

